@@ -32,9 +32,59 @@ except NameError:
     pass
 
 
-class ContextObject(object):
-    """Object passed along Click context"""
+class FeatureGroup(click.Group):
+    """Custom group to handle hiding of commands based on the `feature` tag
+    TODO: Deprecate once Click 7 lands and use `hidden` parameter in commands
+    """
+    def __init__(self, **attrs):
+        click.Group.__init__(self, **attrs)
 
+    def format_commands(self, ctx, formatter):
+        """Custom formatter to control whether a command is displayed
+        Note: This is only called when formatting the help message.
+        """
+        ctx.obj = ContextObject()
+        try:
+            ctx.obj.api = Connection.from_file('~/.transcriptic')
+        except (FileNotFoundError, OSError):
+            # This defaults to feature_groups = []
+            ctx.obj.api = Connection(use_environ=False)
+
+        rows = []
+        for subcommand in self.list_commands(ctx):
+            cmd = self.get_command(ctx, subcommand)
+            if cmd is None:
+                continue
+            try:
+                if cmd.feature is not None and \
+                        cmd.feature in ctx.obj.api.feature_groups:
+                    help = cmd.short_help or ''
+                    rows.append((subcommand, help))
+                else:
+                    continue
+            except AttributeError:
+                help = cmd.short_help or ''
+                rows.append((subcommand, help))
+
+        if rows:
+            with formatter.section('Commands'):
+                formatter.write_dl(rows)
+
+
+class FeatureCommand(click.Command):
+    """Extend off Command to add `feature` attribute
+    TODO: Deprecate once Click 7 lands and use `hidden` parameter in commands
+    """
+    def __init__(self, feature=None, **attrs):
+        click.Command.__init__(self, **attrs)
+        self.feature = feature
+
+
+class ContextObject(object):
+    """Object passed along Click context
+    Note: `ctx` is passed along whenever the @click.pass_context decorator is
+    present. This object is referenced using `ctx.obj`
+    """
     def __init__(self):
         self._api = None
 
@@ -46,10 +96,11 @@ class ContextObject(object):
     def api(self, value):
         self._api = value
 
+
 _CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 
 
-@click.group(context_settings=_CONTEXT_SETTINGS)
+@click.group(context_settings=_CONTEXT_SETTINGS, cls=FeatureGroup)
 @click.option('--apiroot', default=None)
 @click.option(
     '--config',
@@ -61,7 +112,10 @@ _CONTEXT_SETTINGS = dict(help_option_names=['-h', '--help'])
 @click.version_option(prog_name="Transcriptic Python Library (TxPy)")
 @click.pass_context
 def cli(ctx, apiroot, config, organization):
-    """A command line tool for working with Transcriptic."""
+    """A command line tool for working with Transcriptic.
+    Note: This is the main entry point of the CLI. ALl commands go through
+    this `group` before calling a sub-`command`
+    """
     if ctx.invoked_subcommand in ['login', 'compile', 'preview', 'summarize', 'init']:
         # For login/local commands, initialize empty connection
         ctx.obj = ContextObject()
@@ -87,7 +141,7 @@ def cli(ctx, apiroot, config, organization):
             pass
 
 
-@cli.command()
+@cli.command(cls=FeatureCommand, feature='can_submit_autoprotocol')
 @click.argument('file', default='-')
 @click.option(
     '--project', '-p',
@@ -118,11 +172,11 @@ def submit(ctx, file, project, title=None, test=None):
         run_id = req_json['id']
         click.echo("Run created: %s" %
                    ctx.obj.api.url("%s/runs/%s" % (project, run_id)))
-    except Exception as e:
-        click.echo(str(e))
+    except Exception as err:
+        click.echo("\n" + str(err))
 
 
-@cli.command('build-release')
+@cli.command('build-release', cls=FeatureCommand, feature='can_upload_packages')
 @click.argument('package', required=False, metavar="PACKAGE")
 @click.option('--name', '-n', help="Optional name for your zip file")
 @click.pass_context
@@ -158,7 +212,8 @@ def release(ctx, name=None, package=None):
             upload_release, archive=(filename + ".zip"), package=package_id)
 
 
-@cli.command("upload-release")
+@cli.command('upload-release', cls=FeatureCommand,
+             feature='can_upload_packages')
 @click.argument('archive', required=True, type=click.Path(exists=True),
                 metavar="ARCHIVE")
 @click.argument('package', required=True, metavar="PACKAGE")
@@ -177,6 +232,7 @@ def upload_release(ctx, archive, package):
     except AttributeError:
         click.echo("Error: Invalid package id or name.")
         return
+
     with click.progressbar(None, 100, "Upload Progress",
                            show_eta=False, width=70,
                            fill_char="|", empty_char="-") as bar:
@@ -207,14 +263,18 @@ def upload_release(ctx, archive, package):
                 package_id=package_id
             )
             re = up['id']
-        except ValueError:
-            click.echo("\nError: There was a problem uploading your release."
-                       "\nVerify that your manifest.json file is properly  "
-                       "formatted and that all previews in your manifest "
-                       "produce valid Autoprotocol by using the "
-                       "`transcriptic preview` and/or `transcriptic analyze` "
-                       "commands.")
+        except (ValueError, PermissionError) as err:
+            if type(err) == ValueError:
+                click.echo("\nError: There was a problem uploading your release."
+                           "\nVerify that your manifest.json file is properly  "
+                           "formatted and that all previews in your manifest "
+                           "produce valid Autoprotocol by using the "
+                           "`transcriptic preview` and/or `transcriptic analyze` "
+                           "commands.")
+            elif type(err) == PermissionError:
+                click.echo("\n" + str(err))
             return
+
         bar.update(20)
         time.sleep(10)
         status = ctx.obj.api.get_release_status(package_id=package_id, release_id=re,
@@ -238,15 +298,16 @@ def upload_release(ctx, archive, package):
 @cli.command()
 @click.pass_context
 @click.option(
-    '--remote',
+    '--local',
     is_flag=True,
     required=False,
     default=False,
-    help='Shows available protocols to be launched remotely'
+    help='Shows available local protocols instead of remote protocols'
 )
-def protocols(ctx, remote):
+@click.option("--json", "json_flag", help="print JSON response", is_flag=True)
+def protocols(ctx, local, json_flag):
     """List protocols within your manifest or organization."""
-    if remote:
+    if not local:
         protocol_objs = ctx.obj.api.get_protocols()
     else:
         manifest = load_manifest()
@@ -255,15 +316,17 @@ def protocols(ctx, remote):
                        " is improperly formatted.")
             return
         protocol_objs = manifest['protocols']
-
-    click.echo('\n{:^60}'.format("Protocols within this {}:".format("organization" if remote else "manifest")))
-    click.echo('{:-^60}'.format(''))
-    for p in protocol_objs:
-        if p.get('display_name'):
-            display_str = "{} ({})".format(p['name'], p.get('display_name'))
-        else:
-            display_str = p['name']
-        click.echo("{}\n{}".format(display_str, '{:-^60}'.format("")))
+    if json_flag:
+        click.echo(json.dumps(protocol_objs))
+    else:
+        click.echo('\n{:^60}'.format("Protocols within this {}:".format("organization" if not local else "manifest")))
+        click.echo('{:-^60}'.format(''))
+        for p in protocol_objs:
+            if p.get('display_name'):
+                display_str = u"{} ({})".format(p[u'name'], p.get(u'display_name'))
+            else:
+                display_str = p[u'name']
+            click.echo(u"{}\n{}".format(display_str, u'{:-^60}'.format("")))
 
 
 @cli.command()
@@ -315,7 +378,8 @@ def packages(ctx, i):
                 click.echo('{:-^90}'.format(''))
 
 
-@cli.command("create-package")
+@cli.command('create-package', cls=FeatureCommand,
+             feature='can_upload_packages')
 @click.argument('name')
 @click.argument('description')
 @click.pass_context
@@ -328,19 +392,24 @@ def create_package(ctx, description, name):
                        "\"%s\". Please choose a different package name." %
                        name)
             return
-    new_pack = ctx.obj.api.create_package(name, description)
-    if new_pack:
-        click.echo("New package '%s' created with id %s \n"
-                   "View it at %s" % (name, new_pack['id'],
-                                      ctx.obj.api.url('packages/%s' %
-                                                      new_pack['id'])
-                                      )
-                   )
-    else:
-        click.echo("There was an error creating this package.")
+    try:
+        new_pack = ctx.obj.api.create_package(name, description)
+        if new_pack:
+            click.echo(
+                "New package '%s' created with id %s \n"
+                "View it at %s" % (
+                    name, new_pack['id'],
+                    ctx.obj.api.url('packages/%s' % new_pack['id'])
+                )
+            )
+        else:
+            click.echo("There was an error creating this package.")
+    except Exception as err:
+        click.echo("\n" + str(err))
 
 
-@cli.command("delete-package")
+@cli.command('delete-package', cls=FeatureCommand,
+             feature='can_upload_packages')
 @click.argument('name')
 @click.option('--force', '-f', help="force delete a package without being \
               prompted if you're sure", is_flag=True)
@@ -349,24 +418,28 @@ def delete_package(ctx, name, force):
     """Delete an existing protocol package"""
     package_id = get_package_id(name)
     if package_id:
-        if not force:
-            click.confirm(
-                "Are you sure you want to permanently delete the package "
-                "'%s'?  All releases within will be lost." %
-                get_package_name(package_id), default=False, abort=True
-            )
-            click.confirm("Are you really really sure?", default=True)
-        del_pack = ctx.obj.api.delete_package(package_id=package_id)
-        if del_pack:
-            click.echo("Package deleted.")
-        else:
-            click.echo("There was a problem deleting this package.")
+        try:
+            if not force:
+                click.confirm(
+                    "Are you sure you want to permanently delete the package "
+                    "'%s'?  All releases within will be lost." %
+                    get_package_name(package_id), default=False, abort=True
+                )
+                click.confirm("Are you really really sure?", default=True)
+            del_pack = ctx.obj.api.delete_package(package_id=package_id)
+            if del_pack:
+                click.echo("Package deleted.")
+            else:
+                click.echo("There was a problem deleting this package.")
+        except Exception as err:
+            click.echo("\n" + str(err))
 
 
 @cli.command()
 @click.pass_context
 @click.option("-i")
-def projects(ctx, i):
+@click.option("--json", "json_flag", help="print JSON response", is_flag=True)
+def projects(ctx, i, json_flag):
     """List the projects in your organization"""
     try:
         projects = ctx.obj.api.projects()
@@ -378,6 +451,8 @@ def projects(ctx, i):
             all_proj[proj["name"] + status] = proj["id"]
         if i:
             return proj_names
+        elif json_flag:
+            return click.echo(json.dumps(projects))
         else:
             click.echo('\n{:^80}'.format("PROJECTS:\n"))
             click.echo('{:^40}'.format("PROJECT NAME") + "|" +
@@ -395,7 +470,8 @@ def projects(ctx, i):
 @cli.command()
 @click.pass_context
 @click.argument('project_name')
-def runs(ctx, project_name):
+@click.option("--json", "json_flag", help="print JSON response", is_flag=True)
+def runs(ctx, project_name, json_flag):
     """List the runs that exist in a project"""
     project_id = get_project_id(project_name)
     run_list = []
@@ -410,22 +486,30 @@ def runs(ctx, project_name):
                              r['completed_at'].split("T")[0] if r['completed_at']
                              else r['created_at'].split("T")[0],
                              r['status'].replace("_", " ")])
-
-        click.echo(
-            '\n{:^120}'.format("Runs in Project '%s':\n" %
-                               get_project_name(project_name))
-        )
-        click.echo('{:^30}'.format("RUN TITLE") + "|" +
-                   '{:^30}'.format("RUN ID") + "|" +
-                   '{:^30}'.format("RUN DATE") + "|" +
-                   '{:^30}'.format('RUN STATUS'))
-        click.echo('{:-^120}'.format(''))
-        for run in run_list:
-            click.echo('{:^30}'.format(run[0]) + "|" +
-                       '{:^30}'.format(run[1]) + "|" +
-                       '{:^30}'.format(run[2]) + "|" +
-                       '{:^30}'.format(run[3]))
+        if json_flag:
+            return click.echo(json.dumps(map(lambda x: {
+                                 'title': x['title'] or "(Untitled)",
+                                 'id': x['id'],
+                                 'completed_at': x['completed_at']
+                                 if x['completed_at'] else None,
+                                 'created_at': x['created_at'],
+                                 'status': x['status']}, req)))
+        else:
+            click.echo(
+                '\n{:^120}'.format("Runs in Project '%s':\n" %
+                                   get_project_name(project_name))
+            )
+            click.echo('{:^30}'.format("RUN TITLE") + "|" +
+                       '{:^30}'.format("RUN ID") + "|" +
+                       '{:^30}'.format("RUN DATE") + "|" +
+                       '{:^30}'.format('RUN STATUS'))
             click.echo('{:-^120}'.format(''))
+            for run in run_list:
+                click.echo(u'{:^30}'.format(run[0]) + "|" +
+                           u'{:^30}'.format(run[1]) + "|" +
+                           u'{:^30}'.format(run[2]) + "|" +
+                           u'{:^30}'.format(run[3]))
+                click.echo(u'{:-^120}'.format(''))
 
 
 @cli.command("create-project")
@@ -607,7 +691,7 @@ def inventory(ctx, include_aliquots, show_status, retrieve_all, query):
                            "Use flags to modify your search")
 
 
-@cli.command()
+@cli.command(cls=FeatureCommand, feature='can_upload_packages')
 @click.argument('path', default='.')
 def init(path):
     """Initialize a directory with a manifest.json file."""
@@ -639,7 +723,7 @@ def init(path):
         click.echo("manifest.json created")
 
 
-@cli.command()
+@cli.command(cls=FeatureCommand, feature='can_submit_autoprotocol')
 @click.argument('file', default='-')
 @click.option('--test', help='Analyze this run in test mode', is_flag=True)
 @click.pass_context
@@ -661,8 +745,8 @@ def analyze(ctx, file, test):
         analysis = ctx.obj.api.analyze_run(protocol, test_mode=test)
         click.echo(u"\u2713 Protocol analyzed")
         price(analysis)
-    except Exception as e:
-        click.echo(str(e))
+    except Exception as err:
+        click.echo("\n" + str(err))
 
 
 def price(response):
@@ -686,11 +770,12 @@ def price(response):
         click.echo("WARNING (%s): %s" % (context, message))
 
 
-@cli.command()
+@cli.command(cls=FeatureCommand, feature='can_upload_packages')
 @click.argument('protocol_name', metavar="PROTOCOL_NAME")
 @click.option('--view', is_flag=True)
+@click.option('--dye_test', is_flag=True)
 @click.pass_context
-def preview(ctx, protocol_name, view):
+def preview(ctx, protocol_name, view, dye_test):
     """Preview the Autoprotocol output of protocol in the current package."""
     manifest, protocol = load_manifest_and_protocol(protocol_name)
 
@@ -701,7 +786,7 @@ def preview(ctx, protocol_name, view):
                    "contain a \"preview\" section")
         return
 
-    run_protocol(manifest, protocol, inputs, view)
+    run_protocol(manifest, protocol, inputs, view, dye_test)
 
 
 @cli.command()
@@ -719,10 +804,10 @@ def summarize(ctx, file, tree, lookup, runtime):
     A Job Tree refers to a structure of protocol based on container dependency,
     where each node, and its corresponding number, represents an instruction of
     the protocol. More specifically, the tree structure contains process branches,
-    in which the x-axis refers to the dependency depth in a given branch, while 
-    the y-axis refers to the traversal of branches themselves. 
+    in which the x-axis refers to the dependency depth in a given branch, while
+    the y-axis refers to the traversal of branches themselves.
 
-    Example usage is as follows: 
+    Example usage is as follows:
 
     python my_script.py | transcriptic summarize --tree
 
@@ -772,7 +857,7 @@ def summarize(ctx, file, tree, lookup, runtime):
             print("\nYour Job Tree is complete!\n")
 
 
-@cli.command()
+@cli.command(cls=FeatureCommand, feature='can_upload_packages')
 @click.argument('protocol_name', metavar="PROTOCOL_NAME")
 @click.argument('args', nargs=-1)
 def compile(protocol_name, args):
@@ -803,30 +888,29 @@ def compile(protocol_name, args):
     '--project', '-p',
     metavar='PROJECT_ID',
     required=False,
-    help='Project id or name context for configuring the protocol. Use \
-         `transcriptic projects` command to list existing projects.'
+    help='Project id or name context for configuring the protocol. Use '
+         '`transcriptic projects` command to list existing projects.'
 )
 @click.option(
     '--save_input',
     metavar='FILE',
     required=False,
-    help='Save the protocol or parameters input JSON in a file. This is \
-          useful for debugging a protocol.'
+    help='Save the protocol or parameters input JSON in a file. This is '
+          'useful for debugging a protocol.'
 )
 @click.option(
-    '--remote',
+    '--local',
     is_flag=True,
     required=False,
-    help='If specified, the protocol will execute remotely and submit a run. This is useful \
-          if you do not have the protocol locally, or are unable to execute it for any reason.'
+    help='If specified, the protocol will launch a local protocol and submit a run.'
 )
 @click.pass_context
-def launch(ctx, protocol, project, save_input, remote, params):
+def launch(ctx, protocol, project, save_input, local, params):
     """Configure and launch a protocol either using the local manifest file or remotely.
     If no parameters are specified, uses the webapp to select the inputs."""
 
     # Load protocol from local file if not remote and load from listed protocols otherwise
-    if not remote:
+    if local:
         manifest, protocol_obj = load_manifest_and_protocol(protocol)
     else:
         print_stderr("Searching for {}...".format(protocol))
@@ -862,7 +946,7 @@ def launch(ctx, protocol, project, save_input, remote, params):
             except Exception as e:
                 print_stderr("\nUnable to save inputs: %s" % str(e))
 
-    if remote:
+    if not local:
         # For remote execution, use input params file if specified, else use quick_launch inputs
         if not params:
             params = dict(parameters=quick_launch["raw_inputs"])
@@ -881,17 +965,29 @@ def launch(ctx, protocol, project, save_input, remote, params):
                              "File is probably incorrectly formatted.")
                 return
 
-        launch_protocol = _get_launch_request(ctx, params, protocol_obj)
+        req_id, launch_protocol = _get_launch_request(ctx, params, protocol_obj)
+
+        # Check for generation errors
+        generation_errs = launch_protocol["generation_errors"]
+        if len(generation_errs) > 0:
+            for errors in generation_errs:
+                click.echo("\n\n" + errors["message"])
+            click.echo("\nPlease fix the above errors and try again.")
+            return
+
         from time import strftime, gmtime
         default_title = "{}_{}".format(protocol, strftime("%b_%d_%Y", gmtime()))
+
         try:
-            req_json = ctx.obj.api.submit_run(
-                launch_protocol["autoprotocol"], project_id=project, title=default_title, test_mode=None)
+            req_json = ctx.obj.api.submit_launch_request(
+                req_id, protocol_id=protocol_obj["id"],
+                project_id=project, title=default_title, test_mode=None
+            )
             run_id = req_json['id']
             click.echo("\nRun created: %s" %
                        ctx.obj.api.url("%s/runs/%s" % (project, run_id)))
-        except Exception as e:
-            click.echo(str(e))
+        except Exception as err:
+            click.echo("\n" + str(err))
     else:
         print_stderr("\nGenerating Autoprotocol....\n")
         if not params:
@@ -918,7 +1014,7 @@ def _get_launch_request(ctx, params, protocol):
 
     # Wait until launch request is updated (max 5 minutes)
     count = 1
-    while count <= 150 and launch_protocol['autoprotocol'] is None:
+    while count <= 150 and launch_protocol['progress'] != 100:
         sys.stderr.write(
             "\rWaiting for launch request to be configured%s" % ('.' * count))
         sys.stderr.flush()
@@ -927,7 +1023,7 @@ def _get_launch_request(ctx, params, protocol):
                                                          launch_request_id=launch_request_id)
         count += 1
 
-    return launch_protocol
+    return launch_request_id, launch_protocol
 
 
 def _get_quick_launch(ctx, protocol, project):
@@ -1037,6 +1133,7 @@ def login(ctx, api_root, analytics=True):
     token = (user.get('authentication_token') or
              user['test_mode_authentication_token'])
     user_id = user.get("id")
+    feature_groups = user.get('feature_groups')
     if len(user['organizations']) < 1:
         click.echo("Error: You don't appear to belong to any organizations. \n"
                    "Visit %s and create an organization." % api_root)
@@ -1078,7 +1175,8 @@ def login(ctx, api_root, analytics=True):
         sys.exit(1)
     ctx.obj.api = Connection(email=email, token=token,
                              organization_id=organization, api_root=api_root,
-                             user_id=user_id, analytics=analytics)
+                             user_id=user_id, analytics=analytics,
+                             feature_groups=feature_groups)
     ctx.obj.api.save(ctx.parent.params['config'])
     click.echo('Logged in as %s (%s)' % (user['email'], organization))
 
@@ -1181,7 +1279,7 @@ def load_manifest_and_protocol(ctx, protocol_name):
 
 
 @click.pass_context
-def run_protocol(ctx, manifest, protocol, inputs, view=False):
+def run_protocol(ctx, manifest, protocol, inputs, view=False, dye_test=False):
     try:
         command = protocol['command_string']
     except KeyError:
@@ -1196,7 +1294,10 @@ def run_protocol(ctx, manifest, protocol, inputs, view=False):
         fp.write(bytes(json.dumps(inputs), 'UTF-8'))
         fp.flush()
         try:
-            protocol = check_output(["bash", "-c", command + " " + fp.name])
+            if dye_test:
+                protocol = check_output(["bash", "-c", command + " " + fp.name + " --dye_test"])
+            else:
+                protocol = check_output(["bash", "-c", command + " " + fp.name])
             click.echo(protocol)
             if view:
                 click.echo("View your protocol's raw JSON above or see the "
@@ -1207,7 +1308,7 @@ def run_protocol(ctx, manifest, protocol, inputs, view=False):
             return
 
 
-@cli.command()
+@cli.command(cls=FeatureCommand, feature='can_upload_packages')
 @click.argument('manifest', default='manifest.json')
 def format(manifest):
     """Check Autoprotocol format of manifest.json."""
