@@ -5,13 +5,11 @@ import transcriptic
 import os
 from os.path import expanduser
 from . import routes
-from autoprotocol import Protocol
 import requests
 from .version import __version__
 import platform
 import inspect
 
-
 class Connection(object):
     """
     A Connection object is the object used for communicating with Transcriptic.
@@ -66,7 +64,8 @@ class Connection(object):
     """
     def __init__(self, email=None, token=None, organization_id=False,
                  api_root="https://secure.transcriptic.com", organization=False,
-                 cookie=False, verbose=False, use_environ=True, analytics=True, user_id="default"):
+                 cookie=False, verbose=False, use_environ=True, analytics=True,
+                 user_id="default", feature_groups=[]):
         if email is None and use_environ:
             email = os.environ['USER_EMAIL']
             token = os.environ['USER_TOKEN']
@@ -78,6 +77,8 @@ class Connection(object):
         self.verbose = verbose
         self.analytics = analytics
         self.user_id = user_id
+        RELEVANT_GROUPS = set(['can_submit_autoprotocol', 'can_upload_packages'])
+        self.feature_groups = list(RELEVANT_GROUPS.intersection(feature_groups))
         self.headers = {
             "X-User-Email": email,
             "X-User-Token": token,
@@ -107,7 +108,7 @@ class Connection(object):
             return Connection(**cfg)
 
     def save(self, path):
-        """Saves current connection into file"""
+        """Saves current connection into specified file, used for CLI"""
         with open(expanduser(path), 'w') as f:
             f.write(json.dumps({
                 'email': self.email,
@@ -115,7 +116,8 @@ class Connection(object):
                 'organization_id': self.organization_id,
                 'api_root': self.api_root,
                 'analytics': self.analytics,
-                'user_id': self.user_id
+                'user_id': self.user_id,
+                'feature_groups': self.feature_groups
             }, indent=2))
 
     def update_environment(self, **kwargs):
@@ -132,10 +134,10 @@ class Connection(object):
     def preview_protocol(self, protocol):
         """Post protocol preview"""
         route = self.get_route('preview_protocol')
+        protocol = _parse_protocol(protocol)
         return self.post(route,
                          json={
-                             "protocol": json.dumps(protocol.as_dict()) if
-                             isinstance(protocol, Protocol) else protocol
+                             "protocol": json.dumps(protocol)
                          },
                          allow_redirects=False,
                          status_response={
@@ -151,7 +153,11 @@ class Connection(object):
     def get_organization(self, org_id=None):
         """Get particular organization"""
         route = self.get_route('get_organization', org_id=org_id)
-        resp = self.get(route, status_response={'200': lambda resp: resp, 'default': lambda resp: resp})
+        resp = self.get(route, status_response={
+            '200': lambda resp: resp,
+            '404': lambda resp: RuntimeError('There was an error fetching the organization {}'.format(org_id)),
+            'default': lambda resp: resp
+        })
         return resp
 
     def projects(self):
@@ -250,11 +256,31 @@ class Connection(object):
         route = self.get_route('create_quick_launch', project_id=project_id)
         return self.post(route, data=data)
 
+    def launch_protocol(self, params, protocol_id=None):
+        """Launch protocol-id with params"""
+        route = self.get_route('launch_protocol', protocol_id=protocol_id)
+        return self.post(route, data=params)
+
+    def get_launch_request(self, protocol_id=None, launch_request_id=None):
+        """Get launch request id"""
+        route = self.get_route('get_launch_request', protocol_id=protocol_id, launch_request_id=launch_request_id)
+        return self.get(route)
+
+    def get_protocols(self):
+        """Get list of available protocols"""
+        route = self.get_route('get_protocols')
+        return self.get(route)
+
     def resources(self, query):
         """Get resources"""
         route = self.get_route('query_resources', query=query)
         return self.get(route)
 
+    def inventory(self, query, timeout=30.0, page=0):
+        """Get inventory"""
+        route = self.get_route('query_inventory', query=query, page=page)
+        return self.get(route, timeout=timeout)
+
     def kits(self, query):
         """Get kits"""
         route = self.get_route('query_kits', query=query)
@@ -280,8 +306,7 @@ class Connection(object):
 
     def analyze_run(self, protocol, test_mode=False):
         """Analyze given protocol"""
-        if isinstance(protocol, Protocol):
-            protocol = protocol.as_dict()
+        protocol = _parse_protocol(protocol)
         if "errors" in protocol:
             raise AnalysisException(("Error%s in protocol:\n%s" %
                                      (("s" if len(protocol["errors"]) > 1 else ""),
@@ -303,8 +328,7 @@ class Connection(object):
 
     def submit_run(self, protocol, project_id=None, title=None, test_mode=False):
         """Submit given protocol"""
-        if isinstance(protocol, Protocol):
-            protocol = protocol.as_dict()
+        protocol = _parse_protocol(protocol)
         return self.post(self.get_route('submit_run', project_id=project_id),
                          data=json.dumps({
                              "title": title,
@@ -319,6 +343,28 @@ class Connection(object):
                              '422': lambda resp: AnalysisException("Error creating run: %s" % resp.text)
                          })
 
+    def submit_launch_request(self, launch_request_id, project_id=None,
+                              protocol_id=None, title=None, test_mode=False):
+        """Submit specified launch request"""
+        return self.post(
+            self.get_route('submit_launch_request', project_id=project_id),
+            data=json.dumps({
+                "title": title,
+                "launch_request_id": launch_request_id,
+                "protocol_id": protocol_id,
+                "test_mode": test_mode
+            }),
+            status_response={
+                '404': lambda resp: AnalysisException(
+                    "Error: Couldn't create run (404). \n"
+                    "Are you sure the project %s "
+                    "exists, and that you have access to it?" %
+                    self.url(project_id)),
+                '422': lambda resp: AnalysisException(
+                     "Error creating run: %s" % resp.text)
+            }
+        )
+
     def dataset(self, data_id, key="*"):
         """Get dataset with given data_id"""
         route = self.get_route('dataset', data_id=data_id, key=key)
@@ -328,7 +374,8 @@ class Connection(object):
         """Get datasets belonging to run"""
         route = self.get_route('datasets', project_id=project_id, run_id=run_id)
         return self.get(route, status_response={
-            '404': lambda resp: Exception("[404] No run found for ID " + id)
+            '404': lambda resp: Exception("[404] No run found for ID {}. Please ensure you have the "
+                                          "right permissions.".format(run_id))
         }, timeout=timeout)
 
     def get_zip(self, data_id, file_path=None):
@@ -434,8 +481,12 @@ class Connection(object):
                                          **status_response)
 
     def _handle_response(self, response, **kwargs):
+        unauthorized_resp = "You are not authorized to execute this command. For more information on access " \
+                            "permissions see the package documentation."
         default_status_response = {'200': lambda resp: resp.json(),
                                    '201': lambda resp: resp.json(),
+                                   '401': lambda resp: PermissionError("[%d] %s" % (resp.status_code, unauthorized_resp)),
+                                   '403': lambda resp: PermissionError("[%d] %s" % (resp.status_code, unauthorized_resp)),
                                    'default': lambda resp: Exception("[%d] %s" % (resp.status_code, resp.text))
                                    }
         if kwargs['merge_status']:
@@ -444,7 +495,8 @@ class Connection(object):
         else:
             kwargs.pop('merge_status')
             status_response = dict(**kwargs)
-        return_val = status_response.get(str(response.status_code), default_status_response['default'])
+
+        return_val = status_response.get(str(response.status_code), status_response['default'])
 
         if isinstance(return_val(response), Exception):
             raise return_val(response)
@@ -465,3 +517,15 @@ class AnalysisException(Exception):
 
     def __str__(self):
         return self.message
+
+
+def _parse_protocol(protocol):
+    if isinstance(protocol, dict):
+        return protocol
+    try:
+        from autoprotocol import Protocol
+    except ImportError as IE:
+        raise RuntimeError("Please install `autoprotocol-python` in order to work with Protocol objects")
+    if isinstance(protocol, Protocol):
+        return protocol.as_dict()
+
