@@ -5,7 +5,6 @@ from builtins import str
 import pandas as pd
 from builtins import object
 import warnings
-from autoprotocol import Unit
 from requests.exceptions import ReadTimeout
 from copy import deepcopy
 
@@ -391,8 +390,12 @@ class Dataset(_BaseObject):
         self.data_type = self.attributes["data_type"]
         self._raw_data = None
         self._data = pd.DataFrame()
-        self.container = Container(self.attributes["container"]["id"], attributes=self.attributes["container"],
-                                   connection=connection)
+        try:
+            self.container = Container(self.attributes["container"]["id"],
+                                       attributes=self.attributes["container"],
+                                       connection=connection)
+        except KeyError as e:
+            warnings.warn("Missing key {} when initializing dataset".format(e))
 
     @property
     def raw_data(self):
@@ -642,13 +645,17 @@ class Container(_BaseObject):
 
     def _parse_container_type(self):
         """Helper function for parsing container string into container object"""
-        from autoprotocol.container_type import _CONTAINER_TYPES
+
         container_type = self.attributes["container_type"]
 
         # Return the corresponding AP-Py container object for now. In the future, consider merging
         # the current and future dictionary when instantiating container_type
         try:
+            from autoprotocol.container_type import _CONTAINER_TYPES
             return _CONTAINER_TYPES[container_type["shortname"]]
+        except ImportError:
+            raise warnings.warn("Please install `autoprotocol-python` in order to get container types")
+            return None
         except KeyError:
             warnings.warn("ContainerType given is not supported yet in AP-Py")
             return None
@@ -663,9 +670,16 @@ class Container(_BaseObject):
         """
         if self._aliquots.empty:
             aliquot_list = self.attributes["aliquots"]
-            self._aliquots = pd.DataFrame(sorted([dict({'Well Index': x['well_idx'], 'Name': x['name'], 'Id': x['id'],
+            try:
+                from autoprotocol import Unit
+                self._aliquots = pd.DataFrame(sorted([dict({'Well Index': x['well_idx'], 'Name': x['name'], 'Id': x['id'],
                                                  'Volume': Unit(float(x['volume_ul']), 'microliter')}, **x['properties'])
                                            for x in aliquot_list], key=itemgetter('Well Index')))
+            except ImportError:
+                warnings.warn("Volume is not cast into Unit-type. Please install `autoprotocol-python` in order to have automatic Unit casting")
+                self._aliquots = pd.DataFrame(sorted([dict({'Well Index': x['well_idx'], 'Name': x['name'], 'Id': x['id'],
+                                                 'Volume': float(x['volume_ul'])}, **x['properties'])
+                                           for x in aliquot_list], key=itemgetter('Well Index')))
             indices = self._aliquots.pop('Well Index')
             self._aliquots.set_index(indices, inplace=True)
         return self._aliquots
